@@ -296,6 +296,10 @@ module PuppetDBExtensions
     return test_config[:os_families].has_key? 'debian11-64-1'
   end
 
+  def is_bookworm()
+    return test_config[:os_families].has_key? 'debian12-64-1'
+  end
+
   def is_el8()
     return test_config[:os_families].has_key?('redhat8-64-1') ||
            test_config[:os_families].has_key?('centos8-64-1')
@@ -340,7 +344,11 @@ module PuppetDBExtensions
       # always use the non-nightly version to install released packages
       sanitized_version
     when :upgrade_oldest
-      :puppet7
+      if is_bookworm
+        :puppet8
+      else
+        :puppet7
+      end
     end
   end
 
@@ -353,6 +361,8 @@ module PuppetDBExtensions
       '7.14.0'
     elsif is_bullseye
       '7.9.0'
+    elsif is_bookworm
+      '8.8.0'
     else
       '7.3.1'
     end
@@ -426,15 +436,15 @@ module PuppetDBExtensions
       "#{version}buster"
     elsif host['platform'].include?('debian-11')
       "#{version}bullseye"
+    elsif host['platform'].include?('debian-12')
+      "#{version}bookworm"
     else
       raise ArgumentError, "Unsupported platform: '#{host['platform']}'"
     end
   end
 
   def install_puppetdb_module(hosts, puppet_platform)
-    # While we sort out a new puppetlabs-puppetdb module release, point to a branch that allows us to take the latest puppetlabs-postgresql module
-    on(hosts, 'curl -L https://github.com/puppetlabs/puppetlabs-puppetdb/archive/refs/heads/bump-postgres.tar.gz --output /tmp/puppetlabs-puppetdb')
-    on(hosts, 'puppet module install /tmp/puppetlabs-puppetdb')
+    on(hosts, 'puppet module install puppetlabs-puppetdb')
   end
 
   def install_puppetdb(host, version=nil)
@@ -530,8 +540,8 @@ module PuppetDBExtensions
   end
 
   def postgres_manifest
-    # bionic is EOL, so its pgdg repo has been remove
-    manage_package_repo = ! is_bionic
+    # For EOL OSes, pgdg removes the package repo
+    manage_package_repo = ! (is_bionic || is_buster)
 
     manifest = <<-EOS
       # create the puppetdb database
@@ -858,6 +868,8 @@ EOS
       # Take new configuration file on debian to prevent tests from hanging
       # when this is used to upgrade puppet
       if get_os_family(database) == :debian
+        # Explicitly install ca-certificates-java before java to avoid error installing java on debian
+        database.install_package('ca-certificates-java')
         database.install_package('puppetserver', '-o Dpkg::Options::="--force-confnew"')
       else
         install_package(database, 'puppetserver')
