@@ -17,11 +17,16 @@
   :within-depth 2
   :reason "maplog checks the logger type at compile time."})
 
+(disable-warning
+ {:linter :constant-test
+  :for-macro 'clojure.core/nil?
+  :if-inside-macroexpansion-of #{'puppetlabs.puppetdb.utils/with-log-mdc}
+  :reason "with-log-mdc removes value (instead of setting it) if it's nil."})
+
 (disable-warning
  {:linter :deprecations
   :symbol-matches
-  #{#"^#'puppetlabs\.kitchensink\.core/cn-for-cert$"
-    #"^#'puppetlabs\.puppetdb\.jdbc/call-with-array-converted-query-rows$"
+  #{#"^#'puppetlabs\.puppetdb\.jdbc/call-with-array-converted-query-rows$"
     #"^#'puppetlabs\.puppetdb\.testutils\.services/call-with-puppetdb-instance$"
     #"^#'puppetlabs\.puppetdb\.testutils\.services/with-puppetdb-instance$"
     #"^#'puppetlabs\.trapperkeeper\.testutils\.logging/atom-appender$"
