@@ -38,13 +38,15 @@
      data may linger in the database. We periodically sweep the
      database, compacting it and performing regular cleanup so we can
      maintain acceptable performance."
-  (:require [clojure.string :as str]
+  (:require [clojure.java.jdbc :as sql]
+            [clojure.string :as str]
             [clojure.tools.logging :as log]
             [metrics.counters :as counters :refer [counter inc!]]
             [metrics.gauges :refer [gauge-fn]]
             [metrics.timers :refer [time! timer]]
             [metrics.reporters.jmx :as jmx-reporter]
             [murphy :refer [try! with-final]]
+            [next.jdbc :as nxt]
             [puppetlabs.i18n.core :refer [trs tru]]
             [puppetlabs.kitchensink.core :as kitchensink]
             [puppetlabs.puppetdb.cli.tk-util :refer [run-tk-cli-cmd]]
@@ -541,6 +543,7 @@
     (log/info (trs "Shut down {0} scheduler" name))
     (let [msg (trs "Unable to shut down {0} scheduler; requesting server shutdown"
                    name)]
+      ;; One of the test-stop-with-blocked-scheduler log dump triggers
       (log/error msg)
       (shut-down-after-scheduler-unresponsive
        #(request-shutdown {:puppetlabs.trapperkeeper.core/exit
@@ -810,35 +813,41 @@
   (when emit-cmd-events?
     (async/>!! cmd-event-ch (queue/make-cmd-event cmdref kind))))
 
-(defn check-schema-version
+(defn stop-unless-valid-schema-version
   [desired-version db _service request-shutdown]
   {:pre [(integer? desired-version)]}
-  (let [schema-version (-> (jdbc/with-transacted-connection db
-                             (jdbc/query "select max(version) from schema_migrations"))
-                           first
-                           :max)
-        stop (fn [msg status]
+  (let [stop (fn stop-after-schema-check-failure [msg status]
+               ;; One of the test-stop-with-blocked-scheduler log dump triggers
                (log/error msg)
-               (request-shutdown {::tk/exit
-                                  {:status status
-                                   :messages [[msg *err*]]}}))]
-    (when-not (= schema-version desired-version)
-      (cond
-        (> schema-version desired-version)
-        (stop (str
-               (trs "Please upgrade PuppetDB: ")
-               (trs "your database contains schema migration {0} which is too new for this version of PuppetDB."
-                    schema-version))
-              (int \M))
-
-        (< schema-version desired-version)
-        (stop (str
-               (trs "Please run PuppetDB with the migrate option set to true to upgrade your database. ")
-               (trs "The detected migration level {0} is out of date." schema-version))
-              (int \m))
-
-        :else
-        (throw (Exception. "Unknown state when checking schema versions"))))))
+               (request-shutdown {::tk/exit {:status status
+                                             :messages [[msg *err*]]}}))
+        res (try
+              (jdbc/with-transacted-connection db
+                (when (jdbc/table-exists? "schema_migrations")
+                  (jdbc/query "select max(version) from schema_migrations")))
+              (catch SQLException ex
+                ex))]
+    (if (instance? SQLException res)
+      (if (= (jdbc/sql-state :sqlclient-unable-to-establish-sqlconnection)
+             (.getSQLState res))
+        (log/info (trs "Unable to read schema version: {0}" (.getMessage res)))
+        (log/error (trs "Unable to read schema version: {0}" (.getMessage res))))
+      (let [ver (-> res first :max)]
+        (cond
+          (= ver desired-version) true
+          (nil? res)
+          (stop (trs "Cannot find the PuppetDB schema migration table") 2)
+          (> ver desired-version)
+          (stop (str
+                 (trs "Please upgrade PuppetDB: ")
+                 (trs "your database contains schema migration {0} which is too new for this version of PuppetDB."
+                      ver))
+                (int \M))
+          (< ver desired-version)
+          (stop (str
+                 (trs "Please run PuppetDB with the migrate option set to true to upgrade your database. ")
+                 (trs "The detected migration level {0} is out of date." ver))
+                (int \m)))))))
 
 (defn init-queue [config send-event! cmd-event-ch shutdown-for-ex]
   (let [stockdir (conf/stockpile-dir config)
@@ -916,8 +925,8 @@
                               (str allocate-at-startup-at-least-mb)))
                (vec (repeatedly allocate-at-startup-at-least-mb
                                 #(long-array (* 1024 128))))) ;; ~1mb
-             (check-schema-version (desired-schema-version)
-                                   db service request-shutdown)
+             (stop-unless-valid-schema-version (desired-schema-version)
+                                               db service request-shutdown)
              (catch InterruptedException _
                (log/info (trs "Schema checker interrupted")))))))
      0 schema-check-interval)))
@@ -932,6 +941,21 @@
         (catch InterruptedException _
           (log/info (trs "Garbage collector interrupted")))))))
 
+(defn analyze-partitioned-tables [db shutdown-for-ex]
+  (with-nonfatal-exceptions-suppressed
+    (with-monitored-execution shutdown-for-ex
+      (try!
+        (jdbc/with-db-connection db
+          (jdbc/with-db-transaction []
+            (let [c (sql/db-connection jdbc/*db*)]
+              (doseq [table ["reports" "resource_events"]]
+                (try!
+                  (nxt/execute! c [(str "analyze " table)])
+                  (catch InterruptedException _
+                    (log/info (trs (str table " analysis interrupted")))))))))
+        (catch Exception ex
+          (log/error ex))))))
+
 (defn start-garbage-collection
   "Starts garbage collection of the databases represented in db-configs"
   [{:keys [clean-lock] :as _context}
@@ -945,7 +969,18 @@
           (schedule-with-fixed-delay sched #(invoke-periodic-gc db cfg request
                                                                 shutdown-for-ex
                                                                 clean-lock lock-status)
-                                     (to-millis interval) (to-millis interval)))))))
+                                     (to-millis interval) (to-millis interval))))
+      ;; pg (up to at least 16) never analyzes partitioned table parents
+      ;; Nearly fixed in 14, but removed before release:
+      ;;   https://www.postgresql.org/about/news/postgresql-14-beta-1-released-2213/
+      ;;   https://www.postgresql.org/about/news/postgresql-14-rc-1-released-2309/
+      ;;
+      ;; Assumes the analysis runs quickly enough to avoid needing any
+      ;; enforced serialization, and to avoid (with the current single
+      ;; threaded executor) ever delaying gc enough to matter.
+      (let [hourly (* 60 60 1000)
+            analyze #(analyze-partitioned-tables db shutdown-for-ex)]
+        (schedule-with-fixed-delay sched analyze 0 hourly)))))
 
 
 (defn database-lock-status []
