@@ -1,15 +1,19 @@
 (ns puppetlabs.puppetdb.jdbc
   "Database utilities"
-  (:require [clojure.java.jdbc :as sql]
-            [clojure.string :as str]
-            [clojure.tools.logging :as log]
-            [puppetlabs.kitchensink.core :as kitchensink]
-            [puppetlabs.puppetdb.time :as pl-time]
-            [puppetlabs.puppetdb.jdbc.internal :refer [limit-result-set!]]
-            [puppetlabs.puppetdb.schema :as pls :refer [defn-validated]]
-            [schema.core :as s]
-            [puppetlabs.i18n.core :refer [trs]])
+  (:require
+   [clojure.java.jdbc :as sql]
+   [clojure.set :refer [map-invert]]
+   [clojure.string :as str]
+   [clojure.tools.logging :as log]
+   [honey.sql :as hsql]
+   [puppetlabs.i18n.core :refer [trs]]
+   [puppetlabs.kitchensink.core :as kitchensink]
+   [puppetlabs.puppetdb.jdbc.internal :refer [limit-result-set!]]
+   [puppetlabs.puppetdb.schema :as pls :refer [defn-validated]]
+   [puppetlabs.puppetdb.time :as pl-time :refer [ephemeral-now-ns]]
+   [schema.core :as s])
   (:import
+   (clojure.lang ExceptionInfo)
    (com.zaxxer.hikari HikariDataSource HikariConfig)
    (java.sql Connection SQLException SQLTransientConnectionException)
    (org.postgresql.util PGobject)))
@@ -19,16 +23,53 @@
 (defn db []
   *db*)
 
+(defn connection
+  "Returns the *db* connection.  Intended for use during the transition to
+  next.jdbc.  Throws if no transaction is open as a safety precaution, given
+  the substantial differences between java.jdbc and next.jdbc's semantics.
+  https://github.com/seancorfield/next-jdbc/blob/5c9d4795e31a71228f99d5c583b40b169921c5a8/doc/migration-from-clojure-java-jdbc.md#transactions"
+  []
+  (when-not (>= (:level *db* 0) 1)
+    (throw (Exception. "No active java.jdbc transaction")))
+  (sql/db-connection *db*))
+
+(def sql-state-map
+  {:admin-shutdown "57P01"
+   :idle-in-transaction-session-timeout "25P03"
+   :idle-session-timeout "57P05"
+   :invalid-regular-expression "2201B"
+   :lock-not-available "55P03"
+   :program-limit-exceeded "54000"
+   :not-in-prerequisite-state "55000"
+   :query-canceled "57014"
+   :serialization-failure "40001"
+   ;; check constraint violation
+   :check-violation "23514"
+   :sqlclient-unable-to-establish-sqlconnection "08001"
+   :undefined-table "42P01"})
+
+(def sql-state-name
+  (let [sql-code-map (map-invert sql-state-map)]
+    (fn [code]
+      (sql-code-map code))))
+
 (defn sql-state [kw-name]
-  (or ({:admin-shutdown "57P01"
-        :invalid-regular-expression "2201B"
-        :program-limit-exceeded "54000"
-        :lock-not-available "55P03"
-        :query-canceled "57014"}
-       kw-name)
+  (or (sql-state-map kw-name)
       (throw (IllegalArgumentException.
               (trs "Requested unknown SQL state")))))
 
+(defn table-exists? [name]
+  (some-> (-> *db* sql/db-connection .getMetaData (.getTables nil nil name nil))
+          .next))
+
+(defn clj-jdbc-termination-ex? [ex]
+  (and (instance? ExceptionInfo ex)
+       (let [{:keys [rollback handling]} (ex-data ex)]
+         (and rollback
+              (instance? SQLException handling)
+              (= (sql-state :admin-shutdown)
+                 (.getSQLState handling))))))
+
 (defmacro with-db-connection [spec & body]
   `(sql/with-db-connection [db# ~spec]
      (binding [*db* db#]
@@ -74,14 +115,21 @@
                              params)
                       {:multi? true}))
 
+(defn do-hsql
+  [honeysql-command]
+  (sql/db-do-prepared *db* true
+                      (hsql/format honeysql-command)))
+
 (defn do-commands-outside-txn [& commands]
   (let [^Connection conn (:connection *db*)
         orig (.getAutoCommit conn)]
     (.setAutoCommit conn true)
     (try
-      (doseq [c commands]
+      (doseq [x commands
+              :let [batch (if (string? x) [x] x)]
+              cmd batch]
         (with-open [s (.createStatement conn)]
-          (.execute s c)))
+          (.execute s cmd)))
       (finally (.setAutoCommit conn orig)))))
 
 (defn insert!
@@ -93,115 +141,8 @@
   ([table columns values]
    (sql/insert! *db* table columns values {})))
 
-;; STOLEN: clojure JDBC functions modified to support an ON CONFLICT clause
-
-(defn- insert-multi-row
-  "Given a table and a list of columns, followed by a list of column
-  value sequences, return a vector of the SQL needed for the insert
-  followed by the list of column value sequences."
-  [table columns values {:keys [on-conflict]}]
-  (let [nc (count columns)
-        vcs (map count values)]
-    (if (not (and (or (zero? nc) (= nc (first vcs))) (apply = vcs)))
-      (throw (IllegalArgumentException.
-              "insert! called with inconsistent number of columns / values"))
-      (into [(str "INSERT INTO " (name table)
-                  (when (seq columns)
-                    (str " ( "
-                         (str/join ", " (map (fn [col] (name col)) columns))
-                         " )"))
-                  " VALUES ( "
-                  (str/join ", " (repeat (first vcs) "?"))
-                  " ) "
-                  (when on-conflict
-                    (str "on conflict " on-conflict)))]
-            values))))
-
-(defn- insert-single-row-on-conflict
-  "Given a table and a map representing a row, return a vector of the
-  SQL needed for the insert followed by the list of column values."
-  [table row {:keys [on-conflict]}]
-  (let [ks (keys row)]
-    (into [(str "INSERT INTO " (name table) " ( "
-                (str/join ", " (map (fn [col] (name col)) ks))
-                " ) VALUES ( "
-                (str/join ", " (repeat (count ks) "?"))
-                " ) "
-                (when on-conflict
-                  (str "ON CONFLICT " on-conflict)))]
-          (vals row))))
-
-(defn- insert-cols!
-  "Given a database connection, a table name, a sequence of columns
-  names, a sequence of vectors of column values, one per row, and an
-  options map, insert the rows into the database."
-  [db table cols values opts]
-  (let [{:keys [transaction?]} (merge {:transaction? true}
-                                      (when (map? db) db)
-                                      opts)
-        sql-params (insert-multi-row table cols values opts)]
-    (if (sql/db-find-connection db)
-      (sql/db-do-prepared db transaction? sql-params {:multi? true})
-      (with-open [con (sql/get-connection db)]
-        (sql/db-do-prepared (sql/add-connection db con) transaction?
-                            sql-params {:multi? true})))))
-
-(defn- multi-insert-helper
-  "Given a (connected) database connection and some SQL
-  statements (for multiple inserts), run a prepared statement on each
-  and return any generated keys.  Note: we are eager so an unrealized
-  lazy-seq cannot escape from the connection."
-  [db stmts opts]
-  (doall (map (fn [row] (sql/db-do-prepared-return-keys db false row opts))
-              stmts)))
-
-(defn- insert-helper
-  "Given a (connected) database connection, a transaction flag and some SQL statements
-  (for one or more inserts), run a prepared statement or a sequence of
-  them."
-  [db transaction? stmts opts]
-  (if transaction?
-    (sql/with-db-transaction [t-db db] (multi-insert-helper t-db stmts opts))
-    (multi-insert-helper db stmts opts)))
-
-(defn- insert-rows!
-  "Given a database connection, a table name, a sequence of rows, and an
-  options map, insert the rows into the database.  Options include
-  on-conflict."
-  [db table rows opts]
-  (let [{:keys [identifiers qualifier transaction?]}
-        (merge {:identifiers str/lower-case :transaction? true}
-               (when (map? db) db)
-               opts)
-        sql-params (map (fn [row]
-                          (when-not (map? row)
-                            (throw (IllegalArgumentException.
-                                    "insert / insert-multi! called with a non-map row")))
-                          (insert-single-row-on-conflict table row opts))
-                        rows)]
-    (if (sql/db-find-connection db)
-      (insert-helper db transaction? sql-params
-                     {:identifiers identifiers :qualifier qualifier})
-      (with-open [con (sql/get-connection db)]
-        (insert-helper (sql/add-connection db con) transaction? sql-params
-                       {:identifiers identifiers :qualifier qualifier})))))
-
-;; END STOLEN FUNCTIONS
-
-(defn insert-multi!
-  "Inserts multiple rows in either map form or lists of columns &
-  values form. The database to use is given by (jdbc/db). Returns a
-  sequence with every inserted row as returned by the database."
-  ;; since clojure.java.jdbc will open a connection even when given an empty
-  ;; rows or values sequence, bypass it here if either of those are empty
-  ([table rows]
-   (sql/insert-multi! *db* table rows))
-  ([table cols-or-rows values-or-opts]
-   (if (map? values-or-opts)
-     (insert-rows! *db* table cols-or-rows values-or-opts)
-     (insert-cols! *db* table cols-or-rows values-or-opts {})))
-  ([table cols values opts]
-   (sql/insert-multi! *db* table cols values opts)))
+(defn insert-multi! [& args]
+  (apply sql/insert-multi! *db* args))
 
 (defn update!
   "Calls clojure.jdbc/update! after adding (jdbc/db) as the first argument."
@@ -297,12 +238,15 @@
   call-with-query-rows, and apply any necessary conversions directly
   to each row.  (Most columns cannot be arrays.)"
   ([query f] (call-with-array-converted-query-rows query {} f))
-  ([[sql & params] {:keys [as-arrays?] :as opts} f]
+  ([[sql & params]
+    {:keys [as-arrays? fetch-size]
+     :or {fetch-size 500} :as opts}
+    f]
    (with-db-transaction []
      (with-open [stmt (.prepareStatement ^Connection (:connection *db*) sql)]
        (doseq [[i param] (map vector (range) params)]
          (.setObject stmt (inc i) param))
-       (.setFetchSize stmt 500)
+       (.setFetchSize stmt fetch-size)
        (let [fix-vals (if as-arrays?
                         #(mapv any-sql-array->vec %)
                         #(kitchensink/mapvals any-sql-array->vec %))]
@@ -414,18 +358,6 @@
       (.setType "text[]")
       (.setValue (str \{ (str/join \, quoted) \})))))
 
-;; Q: move/replace?
-(defn create-json-path-extraction
-  "Given a base json field and a path of keys to traverse, construct the proper
-  SQL query of the form base->'key'->'key2'..."
-  [field path]
-   (str field
-        (when (seq path)
-          (->> (map single-quote path)
-               (str/join "->")
-               ;; prefix for the first arrow in field->'key1'...
-               (str "->")))))
-
 ;; TODO this is no longer used by the main query engine (engine.clj), but it is
 ;; still used by the legacy query engines and should be removed once it is
 ;; handled by honeysql everywhere
@@ -563,7 +495,7 @@
      (fn []
        ~@body)))
 
-(defn ^:dynamic enable-jmx
+(defn ^:dynamic *enable-jmx*
   "This function exists to enable starting multiple PuppetDB instances
   inside a single JVM. Starting up a second instance results in a
   collision exception between JMX beans from the two
@@ -667,7 +599,7 @@
      (some->> read-only? (.setReadOnly config))
      (some->> (or user username) str (.setUsername config))
      (some->> password str (.setPassword config))
-     (some->> metrics-registry (enable-jmx config))
+     (some->> metrics-registry (*enable-jmx* config))
      (HikariDataSource. config))))
 
 (defn pooled-datasource
@@ -703,6 +635,9 @@
 (defn current-database []
   (-> "select current_database();" query-to-vec first :current_database))
 
+(defn current-pid []
+  (-> "select pg_backend_pid();" query-to-vec first :pg_backend_pid))
+
 (defn-validated has-database-privilege? :- s/Bool
   [user db privilege]
   (-> ["select has_database_privilege(?, ?, ?)" user db privilege]
@@ -750,3 +685,32 @@
   (do-commands
    (format "grant connect on database %s to %s"
           (double-quote db) (double-quote role))))
+
+(defn local-timeout-ex?
+  "Returns true if ex is an exception that might have been thrown as a
+  result of the timeouts set by update-local-timeouts."
+  [ex]
+  (let [state (.getSQLState ex)]
+    (case state ;; https://www.postgresql.org/docs/current/errcodes-appendix.html
+      ;; query_canceled
+      ;; idle_in_transaction_session_timeout
+      ("57014" "25P03") true
+      false)))
+
+(defn- ms-until-ns-deadline
+  [deadline-ns]
+  (int (quot (- deadline-ns (ephemeral-now-ns))
+             1000000)))
+
+(defn update-local-timeouts
+  "Sets the local timeouts (idle and statement) to respect
+  deadline-ns unless the deadline has passed, then sets them to
+  min-ms."
+  [deadline-ns min-ms]
+  (assert (pos? min-ms))
+  (when (some-> deadline-ns (not= ##Inf))
+    (let [timeout-ms (ms-until-ns-deadline deadline-ns)
+          timeout-ms (if (pos? timeout-ms) timeout-ms min-ms)]
+      (do-commands
+       (format "set local statement_timeout = %d" timeout-ms)
+       (format "set local idle_in_transaction_session_timeout = %d" timeout-ms)))))
