@@ -10,8 +10,8 @@
             [puppetlabs.puppetdb.time :as time]))
 
 ;; Avg bytes in the generate-package-inventory() package array.
-;; Mean 12 chars package name, 6 chars provider, mean 6 version."
-(def avg-package-weight 25)
+;; Mean 12 chars package name, 4 chars provider, mean 6 version."
+(def avg-package-weight 23)
 
 (defn filter-edges
   ([exp-source-type exp-target-type edges]
@@ -377,7 +377,7 @@
     (is (= 5 (generate/vary-param 5 false 0.25)))
   (testing "random distribution"
     (testing "positive"
-      (is (< 0 (generate/vary-param 5 true 0.25) 10)))
+      (is (<= 0 (generate/vary-param 5 true 0.25) 10)))
     (testing "zero"
       (is (= 0 (generate/vary-param 0 true 0.25))))
     (testing "negative")
