@@ -1,11 +1,15 @@
 (ns puppetlabs.puppetdb.cli.services-test
   (:require [clojure.set :refer [subset?]]
+            [next.jdbc.plan :refer [select-one!]]
             [puppetlabs.http.client.sync :as pl-http]
             [puppetlabs.puppetdb.cli.util :refer [err-exit-status]]
             [puppetlabs.puppetdb.command.constants :as cmd-consts]
             [puppetlabs.puppetdb.lint :refer [ignore-value]]
             [puppetlabs.puppetdb.scf.partitioning
-             :refer [get-temporal-partitions]]
+             :refer [create-resource-events-partition
+                     create-reports-partition
+                     get-temporal-partitions]
+             :as part]
             [puppetlabs.trapperkeeper.testutils.logging
              :refer [with-log-output
                      logs-matching
@@ -19,14 +23,12 @@
                      maybe-check-for-updates
                      query]]
             [puppetlabs.puppetdb.testutils.db
-             :refer [*db* *read-db* clear-db-for-testing! with-test-db
-                     with-unconnected-test-db]]
+             :refer [*db* *read-db* with-test-db with-unconnected-test-db]]
             [puppetlabs.puppetdb.testutils.cli
              :refer [example-certname example-report get-factsets]]
             [puppetlabs.puppetdb.command :refer [enqueue-command]]
             [puppetlabs.puppetdb.config :as conf]
             [puppetlabs.puppetdb.jdbc :as jdbc]
-            [puppetlabs.puppetdb.scf.migrate :refer [initialize-schema]]
             [puppetlabs.puppetdb.scf.storage :as scf-store]
             [puppetlabs.puppetdb.scf.storage-utils :as sutils]
             [puppetlabs.puppetdb.time :as time :refer [now to-string]]
@@ -51,9 +53,12 @@
              :refer [block-until-results default-timeout-ms temp-file change-report-time]]
             [puppetlabs.puppetdb.testutils.queue :as tqueue]
             [clojure.string :as str])
-  (:import [clojure.lang ExceptionInfo]
-           (java.util.concurrent CyclicBarrier TimeUnit)
-           [java.util.concurrent.locks ReentrantLock]))
+  (:import
+   (ch.qos.logback.classic Level)
+   (ch.qos.logback.classic.spi ILoggingEvent)
+   (clojure.lang ExceptionInfo)
+   (java.util.concurrent CyclicBarrier TimeUnit)
+   (java.util.concurrent.locks ReentrantLock)))
 
 (deftest update-checking
   (let [config-map {:global {:product-name "puppetdb"
@@ -326,23 +331,16 @@
 (defn purgeable-nodes [node-purge-ttl]
   (let [horizon (time/to-timestamp (time/ago node-purge-ttl))]
     (jdbc/query-to-vec
-     "select * from certnames where deactivated < ? or expired < ?"
+     "select * from certnames_status where deactivated < ? or expired < ?"
      horizon horizon)))
 
 (deftest node-purge-gc-batch-limit
-  ;; At least with the current code, it should be fine to run all the
-  ;; tests with the same server, i.e. collect-garbage is only affected
-  ;; by its arguments.
-  (with-pdb-with-no-gc
-    (let [config (-> *server* (get-service :DefaultedConfig) conf/get-config)
-          node-purge-ttl (get-in config [:database :node-purge-ttl])
-          deactivation-time (time/to-timestamp (time/ago node-purge-ttl))
-          lock (ReentrantLock.)]
-      (doseq [[limit expected-remaining] [[0 0]
-                                          [7 3]
-                                          [100 0]]]
-        (clear-db-for-testing!)
-        (initialize-schema)
+  (doseq [[limit expected-remaining] [[0 0] [7 3] [100 0]]]
+    (with-pdb-with-no-gc
+      (let [config (-> *server* (get-service :DefaultedConfig) conf/get-config)
+            node-purge-ttl (get-in config [:database :node-purge-ttl])
+            deactivation-time (time/to-timestamp (time/ago node-purge-ttl))
+            lock (ReentrantLock.)]
         (dotimes [i 10]
           (let [name (str "foo-" i)]
             (scf-store/add-certname! name)
@@ -357,37 +355,40 @@
 
 (deftest test-stop-with-blocked-scheduler
   (let [requested-shutdown? (promise)
-        ready-to-go? (promise)]
+        ready-to-go? (promise)
+        notable? (fn notable-blocked-scheduler-event?
+                   [^ILoggingEvent event]
+                   (and (.isGreaterOrEqual (.getLevel event) Level/ERROR)
+                        (let [m (.getMessage event)]
+                          (not (or (str/includes? m "Unable to shut down job scheduler")
+                                   (str/includes? m "Unable to read schema version"))))))]
     (with-redefs [svcs/stop-gc-wait-ms (constantly 10)
                   svcs/shut-down-after-scheduler-unresponsive
                   (fn [f]
                     (deliver requested-shutdown? true)
                     (f))]
-      (with-pdb-with-no-gc
-        (let [pdb (get-service *server* :PuppetDBServer)
-              pool (-> pdb service-context :job-pool)
-              _blocker (schedule pool #(do
-                                         (deliver ready-to-go? true)
-                                         (while (not (try
-                                                       @requested-shutdown?
-                                                       (catch InterruptedException _
-                                                         false)))
-                                           (ignore-value true)))
-                                 0)]
-          (is (= true (deref ready-to-go? default-timeout-ms false)))
-          (tkapp/stop *server*)
-          (is (= true @requested-shutdown?)))))))
+      (binding [svc-utils/*notable-log-event?* notable?]
+        (with-pdb-with-no-gc
+          (let [pdb (get-service *server* :PuppetDBServer)
+                pool (-> pdb service-context :job-pool)]
+            (schedule pool
+                      #(do
+                         (deliver ready-to-go? true)
+                         (while (not (try
+                                       @requested-shutdown?
+                                       (catch InterruptedException _
+                                         false)))
+                           (ignore-value true)))
+                      0)
+            (is (= true (deref ready-to-go? default-timeout-ms false)))
+            (tkapp/stop *server*)
+            (is (= true @requested-shutdown?))))))))
 
 (deftest regular-gc-drops-oldest-partitions-incrementally
   (with-unconnected-test-db
     (let [config (-> (create-temp-config)
                      (assoc :database *db* :read-database *read-db*)
                      (assoc-in [:database :gc-interval] "0.01"))
-          store-report #(sync-command-post (svc-utils/pdb-cmd-url)
-                                           example-certname
-                                           "store report"
-                                           cmd-consts/latest-report-version
-                                           (change-report-time example-report %))
           before-gc (CyclicBarrier. 2)
           after-gc (CyclicBarrier. 2)
           original-periodic-gc svcs/invoke-periodic-gc
@@ -406,10 +407,17 @@
            ;; Wait for the first, full gc to finish.
            (.await before-gc)
            (.await after-gc)
-           (store-report "2011-01-01T12:00:01-03:00")
-           (store-report "2011-01-02T12:00:01-03:00")
-           (let [report-parts (set (get-temporal-partitions "reports"))
-                 event-parts (set (get-temporal-partitions "resource_events"))]
+           (let [date1 (time/wire-datetime->instant "2011-01-01T12:00:01-03:00")
+                 date2 (time/wire-datetime->instant "2011-01-02T12:00:01-03:00")]
+             ;; Create partitions manually, old events partitions are filtered out in storage
+             (create-reports-partition date1)
+             (create-resource-events-partition date1)
+
+             (create-reports-partition date2)
+             (create-resource-events-partition date2))
+
+           (let [report-parts (set (get-temporal-partitions "reports_historical"))
+                 event-parts (set (get-temporal-partitions "resource_events_historical"))]
 
              (is (subset? #{{:table "reports_20110101z", :part "20110101z"}
                             {:table "reports_20110102z", :part "20110102z"}}
@@ -423,17 +431,17 @@
              (.await before-gc)
              (.await after-gc)
              (is (= (->> report-parts (sort-by :table) (drop 1) set)
-                    (set (get-temporal-partitions "reports"))))
+                    (set (get-temporal-partitions "reports_historical"))))
              (is (= (->> event-parts (sort-by :table) (drop 1) set)
-                    (set (get-temporal-partitions "resource_events"))))
+                    (set (get-temporal-partitions "resource_events_historical"))))
 
              ;; Let the gc go and make sure it only drops the oldest partition
              (.await before-gc)
              (.await after-gc)
              (is (= (->> report-parts (sort-by :table) (drop 2) set)
-                    (set (get-temporal-partitions "reports"))))
+                    (set (get-temporal-partitions "reports_historical"))))
              (is (= (->> event-parts (sort-by :table) (drop 2) set)
-                    (set (get-temporal-partitions "resource_events")))))))))))
+                    (set (get-temporal-partitions "resource_events_historical")))))))))))
 
 (deftest partition-gc-clears-queries-blocking-it-from-getting-accessexclusivelocks
   (with-unconnected-test-db
@@ -471,9 +479,11 @@
                (.await after-gc)
                (store-report "2011-01-01T12:00:01-03:00")
                (store-report "2011-01-02T12:00:01-03:00")
+               ;; store a latest report to move the other two into historical partitions
+               (store-report (to-string (now)))
 
-               (let [report-parts (set (get-temporal-partitions "reports"))
-                     event-parts (set (get-temporal-partitions "resource_events"))]
+               (let [report-parts (set (get-temporal-partitions "reports_historical"))
+                     event-parts (set (get-temporal-partitions "resource_events_historical"))]
 
                  (is (subset? #{{:table "reports_20110101z", :part "20110101z"}
                                 {:table "reports_20110102z", :part "20110102z"}}
@@ -532,17 +542,17 @@
                                 count)))
 
                  (is (= (->> report-parts (sort-by :table) (drop 1) set)
-                        (set (get-temporal-partitions "reports"))))
+                        (set (get-temporal-partitions "reports_historical"))))
                  (is (= (->> event-parts (sort-by :table) (drop 1) set)
-                        (set (get-temporal-partitions "resource_events"))))
+                        (set (get-temporal-partitions "resource_events_historical"))))
 
                  (.await before-gc)
                  (.await after-gc)
 
                  (is (= (->> report-parts (sort-by :table) (drop 2) set)
-                        (set (get-temporal-partitions "reports"))))
+                        (set (get-temporal-partitions "reports_historical"))))
                  (is (= (->> event-parts (sort-by :table) (drop 2) set)
-                        (set (get-temporal-partitions "resource_events")))))))))))))
+                        (set (get-temporal-partitions "resource_events_historical")))))))))))))
 
 (deftest initialize-db
   (testing "when establishing migration database connections"
@@ -583,7 +593,7 @@
   (with-test-db
     (let [config (-> (create-temp-config)
                      (assoc :database *db* :read-database *read-db*)
-                     (assoc-in [:database :gc-interval] "0.01"))
+                     (assoc-in [:database :gc-interval] "60"))
           store-report #(sync-command-post (svc-utils/pdb-cmd-url)
                                            example-certname
                                            "store report"
@@ -595,16 +605,117 @@
        (fn []
          (testing "A report from 24 hours ago won't be gc'ed with a report-ttl of 1d"
            (store-report (-> 1 time/days time/ago time/to-string))
+           (store-report (to-string (now)))
            (svcs/sweep-reports! *db* {:incremental? false
                                       :report-ttl (time/parse-period "1d")
                                       :resource-events-ttl (time/parse-period "1d")
                                       :db-lock-status db-lock-status})
-           (is (= 1 (count (jdbc/query ["SELECT * FROM reports"]))))
+           (is (= 1 (count (jdbc/query ["SELECT * FROM reports_historical"]))))
+           (is (= 1 (count (jdbc/query ["SELECT * FROM reports_latest"]))))
            (jdbc/do-commands "DELETE FROM reports"))
          (testing "A report from 48 hours ago will be gc'ed with a report-ttl of 1d"
            (store-report (-> 2 time/days time/ago time/to-string))
+           (store-report (to-string (now)))
            (svcs/sweep-reports! *db* {:incremental? false
                                       :report-ttl (time/parse-period "1d")
                                       :resource-events-ttl (time/parse-period "1d")
                                       :db-lock-status db-lock-status})
-           (is (empty? (jdbc/query ["SELECT * FROM reports"])))))))))
+           (is (= 1 (count (jdbc/query ["SELECT * FROM reports_latest"]))))
+           (is (empty? (jdbc/query ["SELECT * FROM reports_historical"])))
+           (jdbc/do-commands "DELETE FROM reports"))
+
+         ;; These tests are not applicable unless our Postgres version is new enough
+         ;; to support the concurrent partition detach feature.
+         (when (scf-store/detach-partitions-concurrently?)
+           (testing "a partition stuck in the pending state is finalized and removed"
+             (let [old-ts (-> 2 time/days time/ago)
+                   partition-table (format "reports_%s"
+                                           (part/date-suffix (part/to-zoned-date-time (time/to-timestamp old-ts))))
+                   lock-acquired (promise)
+                   partition-pending-detach (promise)]
+               (store-report (time/to-string old-ts))
+               (store-report (to-string (now)))
+
+               (future
+                ;; Create a query that will block the ACCESS EXCLUSIVE lock needed
+                ;; by the second transaction of the concurrent detach below
+                (jdbc/with-transacted-connection *read-db*
+                  (jdbc/with-db-transaction  []
+                    (jdbc/query [(format "select * from %s" partition-table)])
+                    (deliver lock-acquired partition-table)
+
+                    ;; wait for partition detach to fail
+                    @partition-pending-detach)))
+
+               ;; Wait until we are sure that the detach partition operation will be blocked
+               @lock-acquired
+
+               (try
+                 (jdbc/do-commands-outside-txn
+                  "SET statement_timeout = 100"
+                  (format "ALTER TABLE reports_historical DETACH PARTITION %s CONCURRENTLY" partition-table))
+                 (catch java.sql.SQLException _)
+                 (finally
+                   (deliver partition-pending-detach partition-table)
+                   (jdbc/do-commands-outside-txn "SET statement_timeout = 0")))
+
+               (is (= [{:inhdetachpending true}]
+                      (jdbc/query ["select inhdetachpending from pg_catalog.pg_inherits where inhparent = 'reports_historical'::regclass and inhrelid = ?::regclass" partition-table])))
+
+               (svcs/sweep-reports! *db* {:incremental? false
+                                          :report-ttl (time/parse-period "1d")
+                                          :resource-events-ttl (time/parse-period "1d")
+                                          :db-lock-status db-lock-status})
+
+               (is (empty?
+                    (jdbc/query ["SELECT tablename FROM pg_tables WHERE tablename = ?" partition-table])))
+
+               (jdbc/do-commands "DELETE FROM reports")))
+
+           (testing "a detached partition that was not removed is cleaned up by gc"
+             (let [old-ts (-> 2 time/days time/ago)
+                   partition-table (format "reports_%s"
+                                           (part/date-suffix (part/to-zoned-date-time (time/to-timestamp old-ts))))]
+               (store-report (time/to-string old-ts))
+               (store-report (to-string (now)))
+
+               ;; Strand the partition before calling GC
+               (jdbc/do-commands-outside-txn
+                (format "ALTER TABLE reports_historical DETACH PARTITION %s CONCURRENTLY" partition-table))
+
+               (svcs/sweep-reports! *db* {:incremental? false
+                                          :report-ttl (time/parse-period "1d")
+                                          :resource-events-ttl (time/parse-period "1d")
+                                          :db-lock-status db-lock-status})
+
+               (is (empty?
+                    (jdbc/query ["SELECT tablename FROM pg_tables WHERE tablename = ?" partition-table])))
+
+               (jdbc/do-commands "DELETE FROM reports")))))))))
+
+(deftest reports-analysis
+  ;; For now, just test for the initial invocation
+  (let [start (now)
+        is-analyzed
+        (fn is-analyzed
+          ([table] (is-analyzed table 0))
+          ([table i]
+           (let [r (jdbc/with-db-transaction []
+                     (->> [(str "select last_analyze, last_autoanalyze"
+                                "  from pg_stat_user_tables where relname = '" table "'")]
+                          (select-one! (jdbc/connection) [:last_analyze :last_autoanalyze])))
+                 last-ms (some-> r :last_analyze .getTime time/from-long)]
+             (if (and last-ms (nil? (:last_autoanalyze r)))
+               (do
+                 (is (= nil (:last_autoanalyze r)))
+                 (is (time/after? last-ms start)))
+               (if (= i 100)
+                 (is false (str table " was eventually analyzed"))
+                 (do
+                   (Thread/sleep 100)
+                   (is-analyzed table (inc i))))))))]
+    (svc-utils/with-puppetdb-instance
+      (doseq [parent ["reports" "reports_historical" "reports_latest"
+                      "resource_events" "resource_events_historical" "resource_events_latest"]]
+        (testing (str parent " analysis times")
+          (is-analyzed parent))))))
