@@ -1,21 +1,24 @@
 (ns puppetlabs.puppetdb.http.nodes-test
-  (:require [cheshire.core :as json]
-            [clojure.test :refer :all]
-            [clojure.string :refer [starts-with?]]
-            [puppetlabs.kitchensink.core :refer [keyset]]
-            [puppetlabs.puppetdb.testutils :refer [paged-results] :as tu]
-            [puppetlabs.puppetdb.testutils.http
-             :refer [*app*
-                     are-error-response-headers
-                     deftest-http-app
-                     convert-response
-                     ordered-query-result
-                     query-result
-                     vector-param
-                     query-response]]
-            [puppetlabs.puppetdb.testutils.nodes :refer [store-example-nodes]]
-            [puppetlabs.puppetdb.query-eng :refer [munge-fn-hook]]
-            [flatland.ordered.map :as omap])
+  (:require
+   [cheshire.core :as json]
+   [clojure.string :as str :refer [starts-with?]]
+   [clojure.test :refer :all]
+   [flatland.ordered.map :as omap]
+   [puppetlabs.kitchensink.core :refer [keyset]]
+   [puppetlabs.puppetdb.query-eng :refer [munge-fn-hook]]
+   [puppetlabs.puppetdb.testutils :refer [paged-results] :as tu]
+   [puppetlabs.puppetdb.testutils.http
+    :refer [*app*
+            are-error-response-headers
+            deftest-http-app
+            convert-response
+            ordered-query-result
+            query-result
+            vector-param
+            query-response]]
+   [puppetlabs.puppetdb.testutils.log :refer [notable-pdb-event?]]
+   [puppetlabs.puppetdb.testutils.nodes :refer [store-example-nodes]]
+   [puppetlabs.trapperkeeper.testutils.logging :refer [with-log-suppressed-unless-notable]])
   (:import
    (java.net HttpURLConnection)))
 
@@ -105,13 +108,6 @@
       (is-query-result' ["~" "certname" "\\w+.example.com"] [db puppet web1 web2])
       (is-query-result' ["~" "certname" "example.net"] []))
 
-    (testing "query that generates 0x00 byte error does not show the error type in the response"
-      (let [certname (str "host-1" (char 0))
-            {:keys [body status]} (query-response method endpoint ["=" "certname" certname])]
-         (is (= 500 status))
-         ;; pg-14 adds additional information onto the end
-         (is (starts-with? body "ERROR: invalid byte sequence for encoding \"UTF8\": 0x00"))))
-
     (testing "querying on latest report hash works"
       (let [cert-hashes (query-result method endpoint ["extract"
                                                        ["certname" "latest_report_hash"]
@@ -123,7 +119,9 @@
     (testing "querying on latest report status works"
       (is-query-result' ["=" "latest_report_status" "success"] [])
       (is-query-result' ["=" "latest_report_status" "failure"] [])
-      (is-query-result' ["=" "latest_report_status" "unchanged"] [web1 db puppet]))
+      (is-query-result' ["=" "latest_report_status" "unchanged"] [web1 db puppet])
+      (is (= #{{:latest_report_status nil} {:latest_report_status "unchanged"}}
+             (query-result method endpoint ["extract" ["latest_report_status"]]))))
 
     (testing "querying on latest report noop works"
       (is-query-result' ["=" "latest_report_noop" true] [])
@@ -158,6 +156,23 @@
       (is-query-result' ["~" ["fact" "ipaddress"] "192.168.1.11\\d"] [db puppet])
       (is-query-result' ["~" ["fact" "hostname"] "web\\d"] [web1 web2]))))
 
+(deftest-http-app query-with-unicode-null
+  [[_version endpoint] endpoints
+   method [:get :post]]
+  ;; Actual message (pg-14 extended it):
+  ;;   ERROR: invalid byte sequence for encoding "UTF8": 0x00
+  ;;     Where: portal "C_1" parameter $1"
+  (testing "does not show the error type in the response"
+    (let [err-msg "ERROR: invalid byte sequence for encoding \"UTF8\": 0x00"
+          notable? #(and (notable-pdb-event? %)
+                         (not (str/includes? (.getMessage %) err-msg)))]
+      (with-log-suppressed-unless-notable notable?
+        (let [certname (str "host-1" (char 0))
+              {:keys [body status]} (query-response method endpoint ["=" "certname" certname])]
+          (is (= 500 status))
+          (is (> 3 (-> body str/split-lines count))) ;; no stack traces
+          (is (starts-with? body err-msg)))))))
+
 (deftest-http-app test-string-coercion-fail
   [[_version endpoint] endpoints
    method [:get :post]]
@@ -660,7 +675,7 @@
               (assoc-in [:headers "content-type"] "application/x-www-form-urlencoded")
               *app*)]
       (is (= 415 status))
-      (is (= "content type application/x-www-form-urlencoded not supported" body))))
+      (is (re-find #"content-type application/x-www-form-urlencoded is not a supported" body))))
   (testing "content type application/json should be accepted"
     (let [{:keys [status] :as response}
           (-> (tu/query-request :post endpoint ["extract" "certname" ["=" "certname" "puppet.example.com"]])
@@ -672,11 +687,15 @@
 (deftest-http-app error-in-query-streaming-is-communicated-to-caller
   [[_version endpoint] endpoints]
   (store-example-nodes)
-  (with-redefs [munge-fn-hook (fn [_] (throw (Exception. "BOOM!")))]
-    (testing "generated-stream thread exceptions make it back to the caller"
-      (let [{:keys [status body]}
-            (-> (tu/query-request :post endpoint ["extract" "certname" ["=" "certname" "puppet.example.com"]])
-                (assoc-in [:headers "content-type"] "application/json")
-                *app*)]
-        (is (= 500 status))
-        (is (= "BOOM!" body))))))
+  (with-log-suppressed-unless-notable #(and (notable-pdb-event? %)
+                                            (not (str/includes? (.getMessage %)
+                                                                "BOOM!")))
+    (with-redefs [munge-fn-hook (fn [_] (throw (Exception. "BOOM!")))]
+      (testing "generated-stream thread exceptions make it back to the caller"
+        (let [{:keys [status body]}
+              (-> (tu/query-request :post endpoint ["extract" "certname"
+                                                    ["=" "certname" "puppet.example.com"]])
+                  (assoc-in [:headers "content-type"] "application/json")
+                  *app*)]
+          (is (= 500 status))
+          (is (= "BOOM!" body)))))))
